@@ -746,7 +746,7 @@ void GeneralTab::formatSection(v4l2_fmtdesc fmt)
 		m_colorspace->addItem("SMPTE 170M", QVariant(V4L2_COLORSPACE_SMPTE170M));
 		m_colorspace->addItem("Rec. 709", QVariant(V4L2_COLORSPACE_REC709));
 		m_colorspace->addItem("sRGB", QVariant(V4L2_COLORSPACE_SRGB));
-		m_colorspace->addItem("Adobe RGB", QVariant(V4L2_COLORSPACE_ADOBERGB));
+		m_colorspace->addItem("opRGB", QVariant(V4L2_COLORSPACE_OPRGB));
 		m_colorspace->addItem("BT.2020", QVariant(V4L2_COLORSPACE_BT2020));
 		m_colorspace->addItem("DCI-P3", QVariant(V4L2_COLORSPACE_DCI_P3));
 		m_colorspace->addItem("SMPTE 240M", QVariant(V4L2_COLORSPACE_SMPTE240M));
@@ -761,7 +761,7 @@ void GeneralTab::formatSection(v4l2_fmtdesc fmt)
 		m_xferFunc->addItem(m_isOutput ? "Default" : "Autodetect", QVariant(V4L2_XFER_FUNC_DEFAULT));
 		m_xferFunc->addItem("Rec. 709", QVariant(V4L2_XFER_FUNC_709));
 		m_xferFunc->addItem("sRGB", QVariant(V4L2_XFER_FUNC_SRGB));
-		m_xferFunc->addItem("Adobe RGB", QVariant(V4L2_XFER_FUNC_ADOBERGB));
+		m_xferFunc->addItem("opRGB", QVariant(V4L2_XFER_FUNC_OPRGB));
 		m_xferFunc->addItem("DCI-P3", QVariant(V4L2_XFER_FUNC_DCI_P3));
 		m_xferFunc->addItem("SMPTE 2084", QVariant(V4L2_XFER_FUNC_SMPTE2084));
 		m_xferFunc->addItem("SMPTE 240M", QVariant(V4L2_XFER_FUNC_SMPTE240M));
@@ -1084,7 +1084,10 @@ bool GeneralTab::createAudioDeviceList()
 
 		void **hint;
 
-		snd_device_name_hint(deviceNum, "pcm", &hint);
+		if (snd_device_name_hint(deviceNum, "pcm", &hint)) {
+			audioDevices--;
+			continue;
+		}
 		for (int i = 0; hint[i] != NULL; i++) {
 			int addAs = addAudioDevice(hint[i], deviceNum);
 			if (addAs == AUDIO_ADD_READ || addAs == AUDIO_ADD_READWRITE)
